<?php
/**
 * Created by PhpStorm.
 * User: usuario1
 * Date: 13/10/2017
 * Time: 10:38 AM
 */

require_once 'dataTableStructureJson.php';
require_once 'lib/customlib/common/date_conversion/DateConversion.php';

class dataTableStructure {

    /** Entity properties */

    private $tableName;
    private $fieldList;
    private $filterFieldList;


    /** Filter properties **/

    private $start;
    private $length;
    private $sortingColsList = array();
    //Filtros:
    private $searchFieldList = array();
    private $filterFieldColumnList = array();
    private $individualFieldFilterFlag = false;


    /** Query properties */

    private $baseTableQuery;
    private $resultQuery;

    private $sSearch = "";
    private $sOrder = "";
    private $sWhere = "";
    private $sLimit = "";


    function __construct($entity) {
        $structure = dataTableStructureJson::getStructure($entity);
        if($structure){
            $this->tableName = $structure->tableName;
            $this->filterFieldList = $structure->filterFieldList;
            $this->setFieldList($structure->fieldList);

            $this->baseTableQuery = "SELECT * FROM ".$this->tableName;
        }
    }

    /** @param string $query */
    public function setBaseTableQuery($query){ $this->baseTableQuery = $query; }
    /** @return string */
    public function getResultQuery(){return $this->resultQuery;}
    /** @param int $start */
    public function setStart($start) { $this->start = $start; }
    /** @param int $length */
    public function setLength($length) { $this->length = $length; }
    /** @return int  */
    public function getStart() { return $this->start; }
    /** @return int */
    public function getLength() { return $this->length; }
    /** @return array asociativo Field => Type */
    public function getFieldList() { return $this->fieldList; }


    public function addSortingCol($column, $direction){
        $this->sortingColsList[$column] = $direction;
    }

    public function addFilterFieldColumn($fieldName, $filterValue){
        $this->filterFieldColumnList[$fieldName] = $filterValue;
        $this->individualFieldFilterFlag = true;
    }

    private function setFieldList($fieldList){
        foreach ($fieldList as $field => $type) {
            $formattedField = new dtTableField($field,$field,$type);
            $this->fieldList[$field] = $formattedField;
        }
    }

    private function getFieldColumnNameFromIndex($i) {
        $arrayKeys = array_keys($this->fieldList);
        return $arrayKeys[$i];
    }


    /****** Confección del Query ******/

    public function getQuery() {
        //$this->setPagingQuery(); //La paginación esta desde el dataTableSql con la funcion setLimit()
        $this->setOrderQuery();
        $this->setSearch();
        $this->setFilterByColumn();

        if ( preg_match("/where/", strtolower($this->baseTableQuery)) )
            $this->resultQuery = " select * from ( $this->baseTableQuery) ax ";
        else
            $this->resultQuery = $this->baseTableQuery;


        $this->resultQuery .= "
          $this->sWhere
          $this->sOrder
          $this->sLimit
        ";

        return $this->resultQuery;
    }

    /**
     * Setea la paginación por medio de query
     */
    private function setPagingQuery(){
        $this->sLimit = "";
        if (isset($this->start) && isset($this->length)) {
            /*
            if ( ereg("WHERE", strtoupper($this->sWhere) ) ) {
                $this->sWhere .= " AND ";
            } else {
                $this->sWhere .= " WHERE ";
            }
            */

            $this->sLimit .= "ROWNUM BETWEEN $this->start and $this->length";
        }
    }

    private function setOrderQuery(){
        if (sizeof($this->sortingColsList) > 0) {
            if ($this->sOrder == "") {
                $this->sOrder = "ORDER BY  ";
                foreach ($this->sortingColsList as $iSortCol => $sSortDir) {

                    if (is_int($iSortCol) )
                        $this->sOrder .= $this->getFieldColumnNameFromIndex($iSortCol) . " $sSortDir , ";
                    else
                        $this->sOrder .= " $iSortCol $sSortDir , ";
                }
                $this->sOrder = substr_replace($this->sOrder, "", -2);
            }
        }
    }

    private function setSearch(){
        $_lower = self::getLowerQueryFunction();

        if ($this->sSearch != "") {

            if (is_array($this->searchFieldList)) {
                if (preg_match("/WHERE/", strtoupper($this->sWhere))) {
                    $this->sWhere .= " AND ";
                } else {
                    $this->sWhere .= " WHERE ";
                }

                foreach ($this->searchFieldList as $field) {
                    $this->sWhere .= " $_lower($field) LIKE '%" . strtolower($this->sSearch) . "%' OR ";
                }
                $this->sWhere = substr_replace($this->sWhere, "", -4);
            }
        }
    }


    private function setFilterByColumn(){
        if ($this->individualFieldFilterFlag) {

            $_lower = self::getLowerQueryFunction();

            if (is_array($this->filterFieldColumnList) and sizeof($this->filterFieldColumnList) > 0) {


                if (preg_match("/WHERE/", strtoupper($this->sWhere))) {
                    $this->sWhere .= " AND ";
                } else {
                    $this->sWhere .= " WHERE ";
                }

                foreach ($this->filterFieldColumnList as $field => $filterValue) {
                    $operando = " = ";
                    $filterValue = trim($filterValue);
                    if ($filterValue !== "") {
                        $_not = "";
                        if (strpos($filterValue,"*") !== false) {
                            $filterValue = str_replace("*", "%", $filterValue);
                            $operando = " LIKE ";
                        }
                        if (substr($filterValue,0,1)==="!") {
                            $operando = " not LIKE ";
                            $filterValue = str_replace("!", " ", $filterValue);
                        }
                        $filterValue = trim($filterValue);

                        $this->sWhere .= " $_lower($field) $operando '" . strtolower($filterValue) . "' AND ";
                    }
                }

                $this->sWhere = substr_replace($this->sWhere, "", -4);
            }
        }
    }


    protected function getLowerQueryFunction(){
        $_ret = "lower";
        if (DbConfig::isOCI8())
            $_ret = "nls_lower";
        return $_ret;
    }

}



class dtTableField {

    const TYPE_DATE = "date";
    const TYPE_DATE_TIME = "datetime";

    private $name;
    private $columnName;
    private $type;
    private $displayFormat = "";     // formato de display para mostrar p.e. fecha dd/mm/aaaa

    function dtTableField($name, $columnName = "", $type = "") {
        $this->name = $name;
        $this->columnName = $name;
        if ($columnName != "")
            $this->columnName = $columnName;
        if($type != "")
            $this->type = $type;
    }

    public function getFormattedValue($value) {
        switch ($this->type){
            case dtTableField::TYPE_DATE:
                $_dateTime = DateConversion::DBFormatToDate($value);
                return DateConversion::DateToViewFormat($_dateTime);
                break;
            case dtTableField::TYPE_DATE_TIME:
                $_dateTime = DateConversion::DBFormatToDatetime($value);
                return DateConversion::DateTimeToViewFormat($_dateTime);
                break;
        }
    }

    public function getName() {
        return $this->name;
    }

    public function setName($name) {
        $this->name = $name;
    }

    public function getColumnName() {
        return $this->columnName;
    }

    public function setColumnName($columnName) {
        $this->columnName = $columnName;
    }

    public function getType() {
        return $this->type;
    }

    public function setType($type) {
        $this->type = $type;
    }

    public function getDisplayFormat() {
        return $this->displayFormat;
    }

    public function setDisplayFormat($displayFormat) {
        $this->displayFormat = $displayFormat;
    }

}


class dtColumnList {

    private $columnList = array();

    public function __construct($get_params) {

        if (sizeof($get_params) > 0 ) {
            foreach ($get_params as $ndx=>$value) {
                $this->columnList[$ndx] = new dtColumn($value);
            }
        }

    }

    public function getColumnList() {
        return $this->columnList;
    }

    /**
     *
     * @param int $ndx
     * @return dtColumn
     */
    public function getColumn($ndx) {
        return $this->columnList[$ndx];
    }

    public function getColumnFilterValue($ndx) {
        $_col =  $this->columnList[$ndx];
        if ($_col)
            return $_col->getSearchValue();
        return false;
    }


}

class dtColumn {

    private $data;
    private $name;
    private $searchable;
    private $orderable;
    private $search;

    public function __construct($get_params) {

        $this->data = $get_params['data'];
        $this->name = $get_params['name'];
        $this->searchable = $get_params['searchable'];
        $this->orderable = $get_params['orderable'];
        $this->search = $get_params['search'];

    }

    public function getName() {
        return $this->name;
    }

    public function getData() {
        return $this->data;
    }

    public function getSearch() {
        return $this->search;
    }

    public function getSearchExpression() {
        return $this->search["value"];
    }

    public function getSearchValue() {
        $_ret = false;

        if ($this->search)
            $_ret = $this->search["value"];

        return $_ret;

    }
}
